/*
 * file CqGroup.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqGroup
 *
 * (C) Copyright IBM Corporation 2006, 2014.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.Feedback;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.StpPrincipal;
import com.ibm.rational.wvcm.stp.StpResource;
import com.ibm.rational.wvcm.stpex.StpExBase;

/**
 * Interface for ClearQuest group resources.
 * <p>
 * Changes you make to a group are immediately reflected in the db-set (schema
 * repository, master database) but not the associated user databases. To update
 * the user databases, apply {@link CqUserDb#doUpgradeUsersAndGroups(Feedback)}
 * on each database individually.
 * <p>
 * The user-friendly specification for the location of a group has the form
 * <pre>
 *  <b>cq.group:</b><i>&lt;group-name&gt;</i>@<i>&lt;db-set&gt;</i>
 * </pre>
 */
public interface CqGroup extends CqResource, StpPrincipal
{
    /** The database set that contains this resource */
    PropertyName<CqDbSet> DB_SET =
        new PropertyName<CqDbSet>(PROPERTY_NAMESPACE, "db-set");

    /**
     * Returns the value of the {@link #DB_SET DB_SET} property as defined by
     * this proxy.
     * 
     * @return A CqDbSet proxy for the database set that contains this resource.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #DB_SET DB_SET} property.
     */
    CqDbSet getDbSet() throws WvcmException;

    /**
     * Indicates whether or not the group is active. This property can be
     * retrieved or set. Members of an inactive group are not allowed to access
     * any databases using the groups attributes. Access to a database is
     * permitted if the user belongs to another group that has access or if the
     * users account is specifically subscribed to the database.
     */
    PropertyName<Boolean> IS_ACTIVE =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-active");

    /**
     * Returns the value of the {@link #IS_ACTIVE IS_ACTIVE} property as defined
     * by this proxy.
     * 
     * @return <b>true</b> if the group is active; <b>false</b> otherwise.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_ACTIVE IS_ACTIVE} property.
     */
    boolean getIsActive() throws WvcmException;

    /**
     * Defines a new value for the {@link #IS_ACTIVE IS_ACTIVE} property of this
     * proxy.
     * 
     * @param value <b>true</b> to make the group active; <b>false</b> to make
     *            it inactive.
     */
    void setIsActive(boolean value);

    /**
     * Returns the databases to which this group may subscribe, including
     * databases to which it is already described.
     */
    PropertyName<ResourceList<CqUserDb>> ALL_DATABASES =
        new PropertyName<ResourceList<CqUserDb>>(PROPERTY_NAMESPACE,
                                                 "all-databases");

    /**
     * Returns the value of the {@link #ALL_DATABASES ALL_DATABASES} property as
     * defined by this proxy.
     * 
     * @return A ResourceList containing a CqUserDb proxy for each database to
     *         which this group may subscribe. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #ALL_DATABASES ALL_DATABASES} property.
     */
    ResourceList<CqUserDb> getAllDatabases() throws WvcmException;

    /**
     * Indicates whether this Group is subscribed to all databases in a Db-set
     * (schema repository, master database).
     */
    PropertyName<Boolean> IS_SUBSCRIBED_TO_ALL_DATABASES =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "is-subscribed-to-all-databases");

    /**
     * Returns the value of the
     * {@link #IS_SUBSCRIBED_TO_ALL_DATABASES IS_SUBSCRIBED_TO_ALL_DATABASES}
     * property as defined by this proxy.
     * 
     * @return <b>true</b> if the Group is subscribed to all databases in the
     *         d, <b>false</b> otherwise.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_SUBSCRIBED_TO_ALL_DATABASES IS_SUBSCRIBED_TO_ALL_DATABASES}
     *             property.
     */
    boolean getIsSubscribedToAllDatabases() throws WvcmException;

    /**
     * Defines a new value for the
     * {@link #IS_SUBSCRIBED_TO_ALL_DATABASES IS_SUBSCRIBED_TO_ALL_DATABASES}
     * property of this proxy.
     * 
     * @param value <b>true</b> if this group is henceforth to be subscribed to
     *            all db-set databases; <b>false</b> if it is to be subscribed
     *            only to the databases on its SUBCRIBED_DATABASES list.
     */
    void setIsSubscribedToAllDatabases(boolean value);

    /**
     * The replica that masters this group
     */
    PropertyName<CqReplica> CQ_MASTER_REPLICA =
        new PropertyName<CqReplica>(PROPERTY_NAMESPACE, "cq-master-replica");

    /**
     * Returns the value of the {@link #CQ_MASTER_REPLICA CQ_MASTER_REPLICA}
     * property as defined by this proxy.
     * 
     * @return A CqReplica proxy for the replica that masters this group
     *         resource. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CQ_MASTER_REPLICA CQ_MASTER_REPLICA} property.
     */
    CqReplica getCqMasterReplica() throws WvcmException;

    /**
     * Defines a new value for the {@link #CQ_MASTER_REPLICA CQ_MASTER_REPLICA}
     * property of this proxy.
     * 
     * @param value A CqReplica proxy for the replica that is to assume
     *            mastership of this group.
     */
    void setCqMasterReplica(CqReplica value);

    /**
     * The user databases to which this group is subscribed. If
     * {@link #getIsSubscribedToAllDatabases()} is true, then this list will
     * include all databases.
     */
    PropertyName<ResourceList<CqUserDb>> SUBSCRIBED_DATABASES =
        new PropertyName<ResourceList<CqUserDb>>(PROPERTY_NAMESPACE,
                                                 "subscribed-databases");

    /**
     * Returns the value of the
     * {@link #SUBSCRIBED_DATABASES SUBSCRIBED_DATABASES} property as defined by
     * this proxy.
     * 
     * @return A ResourceList containing a CqUserDb resource for each user
     *         database to which this group is explicitly or implicitly
     *         subscribed. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #SUBSCRIBED_DATABASES SUBSCRIBED_DATABASES} property.
     */
    ResourceList<CqUserDb> getSubscribedDatabases() throws WvcmException;

    /**
     * Defines a new value for the
     * {@link #SUBSCRIBED_DATABASES SUBSCRIBED_DATABASES} property of this
     * proxy.
     * 
     * @param value A ResourceList containing a CqUserDb proxy for each user
     *            database to which this group is now subscribed. May be <b>null</b>
     *            or empty to unsubscribe the group from all user databases.
     */
    void setSubscribedDatabases(ResourceList<CqUserDb> value);

    /**
     * Establishes the values to be added to and/or deleted from the
     * SUBSCRIBED_DATABASES property when that property is updated from this
     * proxy (via doWriteProperties or any other "do" method of this interface).
     * 
     * @param additions The list of CqUserDb proxies that must be in the
     *            SUBSCRIBED_DATABASES property after it has been updated from
     *            this proxy. Thus, it is OK for a value in this list to already
     *            be a member of the property value. This list may be empty or
     *            null if no values are to be added to the property. A value may
     *            not appear in both this list and the deletions list.
     * 
     * @param deletions The list of CqUserDb proxies that must not be in the
     *            SUBSCRIBED_DATABASES property after it has been updated from
     *            this proxy. Thus, it is OK for a value in this list to not be
     *            a current member of the property value. This list may be empty
     *            or null if no values are to be deleted from the property. A
     *            value may not appear in both this list and the additions list.
     */
    void setSubscribedDatabases(ResourceList<CqUserDb> additions,
                                ResourceList<CqUserDb> deletions);

    /**
     * The users that belong to this group
     */
    PropertyName<ResourceList<CqUser>> USERS =
        new PropertyName<ResourceList<CqUser>>(PROPERTY_NAMESPACE, "users");

    /**
     * Returns the value of the {@link #USERS USERS} property as defined by this
     * proxy. If this property was set via
     * {@link #setSubscribedDatabases(ResourceList, ResourceList)} this method
     * returns only the list of additions.
     * 
     * @return A ResourceList containing a CqUser proxy for each user that is
     *         the member of this group. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #USERS USERS} property.
     */
    ResourceList<CqUser> getUsers() throws WvcmException;

    /**
     * Defines a new value for the {@link #USERS USERS} property of this proxy.
     * 
     * @param value A ResourceList containing a CqUser proxy for each user that
     *            is to be a member of this group.
     */
    void setUsers(ResourceList<CqUser> value);

    /**
     * Establishes the values to be added to and/or deleted from the USERS
     * property when that property is updated from this proxy (via
     * doWriteProperties or any other "do" method of this interface).
     * 
     * @param additions The list of CqUser proxies that must be in the USERS
     *            property after it has been updated from this proxy. Thus, it
     *            is OK for a value in this list to already be a member of the
     *            property value. This list may be empty or null if no values
     *            are to be added to the property. A value may not appear in
     *            both this list and the deletions list.
     * 
     * @param deletions The list of CqUser proxies that must not be in the USERS
     *            property after it has been updated from this proxy. Thus, it
     *            is OK for a value in this list to not be a current member of
     *            the property value. This list may be empty or null if no
     *            values are to be deleted from the property. A value may not
     *            appear in both this list and the additions list.
     */
    void setUsers(ResourceList<CqUser> additions,
                  ResourceList<CqUser> deletions);

    /**
     * The groups that belong to this group
     */
    PropertyName<ResourceList<CqGroup>> GROUPS =
        new PropertyName<ResourceList<CqGroup>>(PROPERTY_NAMESPACE, "groups");

    /**
     * Returns the value of the {@link #GROUPS GROUPS} property as defined by
     * this proxy. If this property was set via
     * {@link #setSubscribedDatabases(ResourceList, ResourceList)} this method
     * returns only the list of additions.
     * 
     * @return A ResourceList containing a CqGroup proxy for each group that is
     *         the member of this group. Will never be <b>null</b>.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #GROUPS GROUPS} property.
     */
    ResourceList<CqGroup> getGroups() throws WvcmException;

    /**
     * Defines a new value for the {@link #GROUPS GROUPS} property of this
     * proxy.
     * 
     * @param value A ResourceList containing a CqGroup proxy for each group
     *            that is to be a member of this group.
     */
    void setGroups(ResourceList<CqGroup> value);

    /**
     * Establishes the values to be added to and/or deleted from the GROUPS
     * property when that property is updated from this proxy (via
     * doWriteProperties or any other "do" method of this interface).
     * 
     * @param additions The list of CqGroup proxies that must be in the GROUPS
     *            property after it has been updated from this proxy. Thus, it
     *            is OK for a value in this list to already be a member of the
     *            property value. This list may be empty or null if no values
     *            are to be added to the property. A value may not appear in
     *            both this list and the deletions list.
     * 
     * @param deletions The list of CqGroup proxies that must not be in the
     *            GROUPS property after it has been updated from this proxy.
     *            Thus, it is OK for a value in this list to not be a current
     *            member of the property value. This list may be empty or null
     *            if no values are to be deleted from the property. A value may
     *            not appear in both this list and the additions list.
     */
    void setGroups(ResourceList<CqGroup> additions,
                   ResourceList<CqGroup> deletions);

    /**
     * Creates a new group and associates it with the DbSet named by the proxy
     * location. The location is of the form
     * 
     * <pre>
     * cq.group:&lt;group-name>@&lt;db-set-name>
     * </pre>
     * 
     * The new group is subscribed to all databases by default. When you use the
     * methods of the Group object to add users and subscribe the group to one
     * or more databases, the groups or users are subscribed only to those you
     * specify.
     * 
     * @param feedback Specifies optional feedback to the caller.
     * @return A proxy for this new resource, whose properties are specified by
     *         feedback.
     * @throws WvcmException ReasonCode:
     *             <li>{@link javax.wvcm.WvcmException.ReasonCode#RESOURCE_ALREADY_EXISTS_AT_LOCATION CANNOT_CREATE_AT_THIS_LOCATION}:
     *             If a resource already exists at the location of this
     *             Resource, the request MUST fail.
     *             <li>{@link javax.wvcm.WvcmException.ReasonCode#CANNOT_CREATE_AT_THIS_LOCATION CANNOT_CREATE_AT_THIS_LOCATION}:
     *             If the location of this Group does not identify a valid
     *             location in which to create a group, the request MUST fail.
     *             Groups must be created in a DbSet
     */
    CqGroup doCreateGroup(Feedback feedback) throws WvcmException;
}
